@@ -3,16 +3,15 @@ import datetime
 import json
 import logging
 import os
-from typing import Dict, List, Optional, Iterable
-
-from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
-
+from argparse import ArgumentDefaultsHelpFormatter, ArgumentParser
 from graphkb import GraphKBConnection
-from graphkb.match import cache_missing_features
+from typing import Dict, Iterable, List, Optional
 
-from . import ipr
 from .annotate import annotate_category_variants, annotate_positional_variants, get_gene_information
+from .connection import IprConnection
+from .constants import DEFAULT_URL
 from .inputs import (
+    check_comparators,
     check_variant_links,
     preprocess_copy_variants,
     preprocess_expression_variants,
@@ -20,9 +19,11 @@ from .inputs import (
     preprocess_structural_variants,
     read_tabbed_file,
 )
+from .ipr import create_key_alterations, filter_structural_variants, select_expression_plots
+from .summary import summarize
+from .therapeutic_options import create_therapeutic_options
 from .types import KbMatch
 from .util import LOG_LEVELS, logger, trim_empty_values
-from .summary import summarize
 
 CACHE_GENE_MINIMUM = 5000
 
@@ -46,7 +47,9 @@ def command_interface() -> None:
         help='username to use connecting to graphkb/ipr',
     )
     parser.add_argument(
-        '--password', required=True, help='password to use connecting to graphkb/ipr',
+        '--password',
+        required=True,
+        help='password to use connecting to graphkb/ipr',
     )
     parser.add_argument('-c', '--copy_variants', required=False, type=file_path)
     parser.add_argument('-m', '--small_mutations', required=False, type=file_path)
@@ -58,13 +61,15 @@ def command_interface() -> None:
         required=True,
         help='Disease name to be used in matching to GraphKB',
     )
-    parser.add_argument('--ipr_url', default=ipr.DEFAULT_URL)
+    parser.add_argument('--ipr_url', default=DEFAULT_URL)
     parser.add_argument('--graphkb_url', default=None)
     parser.add_argument('--log_level', default='info', choices=LOG_LEVELS.keys())
     parser.add_argument('--patient_id', required=True, help='The patient ID for this report')
     parser.add_argument('--project', default='TEST', help='The project to upload this report to')
     parser.add_argument(
-        '-o', '--output_json_path', help='path to a JSON to output the report upload body',
+        '-o',
+        '--output_json_path',
+        help='path to a JSON to output the report upload body',
     )
     parser.add_argument(
         '-w',
@@ -149,6 +154,10 @@ def clean_unsupported_content(upload_content: Dict) -> Dict:
             for col in drop_columns:
                 if col in variant:
                     del variant[col]
+
+    for row in upload_content['kbMatches']:
+        del row['kbContextId']
+        del row['kbRelevanceId']
     return upload_content
 
 
@@ -158,7 +167,7 @@ def create_report(
     patient_id: str,
     kb_disease_match: str,
     project: str = 'TEST',
-    ipr_url: str = ipr.DEFAULT_URL,
+    ipr_url: str = DEFAULT_URL,
     log_level: str = 'info',
     expression_variant_rows: Iterable[Dict] = [],
     structural_variant_rows: Iterable[Dict] = [],
@@ -169,7 +178,6 @@ def create_report(
     always_write_output_json: bool = False,
     ipr_upload: bool = True,
     interactive: bool = False,
-    cache_gene_minimum: int = CACHE_GENE_MINIMUM,
     graphkb_url: str = '',
 ) -> Optional[Dict]:
     """
@@ -201,8 +209,9 @@ def create_report(
     copy_variants = preprocess_copy_variants(copy_variant_rows)
     structural_variants = preprocess_structural_variants(structural_variant_rows)
     expression_variants = preprocess_expression_variants(expression_variant_rows)
+    check_comparators(optional_content or {}, expression_variants)
 
-    ipr_conn = ipr.IprConnection(username, password, ipr_url)
+    ipr_conn = IprConnection(username, password, ipr_url)
     if graphkb_url:
         logger.info(f'connecting to graphkb: {graphkb_url}')
         graphkb_conn = GraphKBConnection(graphkb_url)
@@ -214,13 +223,6 @@ def create_report(
         small_mutations, expression_variants, copy_variants, structural_variants
     )
 
-    # cache of the graphkb gene names speeds up calculation for large
-    # numbers of genes, but has significant overhead and slows down
-    # calculations on small numbers of genes.
-    if len(genes_with_variants) > cache_gene_minimum:
-        logger.info('caching genes to improve matching speed')
-        cache_missing_features(graphkb_conn)
-
     # filter excess variants not required for extra gene information
     logger.info(f'annotating small mutations')
     alterations: List[KbMatch] = annotate_positional_variants(
@@ -257,9 +259,22 @@ def create_report(
 
     output = optional_content or dict()
 
-    key_alterations, variant_counts = ipr.create_key_alterations(
-        alterations, expression_variants + copy_variants + structural_variants + small_mutations
-    )
+    all_variants = expression_variants + copy_variants + structural_variants + small_mutations
+
+    key_alterations, variant_counts = create_key_alterations(alterations, all_variants)
+
+    logger.info('generating therapeutic options')
+    targets = create_therapeutic_options(graphkb_conn, alterations, all_variants)
+
+    logger.info('generating analyst comments')
+    comments = {
+        'comments': summarize(
+            graphkb_conn,
+            alterations,
+            disease_name=kb_disease_match,
+            variants=all_variants,
+        )
+    }
 
     output.update(
         {
@@ -280,42 +295,31 @@ def create_report(
             'kbVersion': timestamp(),
             'structuralVariants': [
                 trim_empty_values(s)
-                for s in ipr.filter_structural_variants(
+                for s in filter_structural_variants(
                     structural_variants, alterations, gene_information
                 )
             ],
             'genes': gene_information,
             'genomicAlterationsIdentified': key_alterations,
             'variantCounts': variant_counts,
+            'analystComments': comments,
+            'therapeuticTarget': targets,
         }
     )
     output.setdefault('images', []).extend(
-        ipr.select_expression_plots(
+        select_expression_plots(
             alterations, expression_variants + copy_variants + structural_variants + small_mutations
         )
     )
-    for section in output:
-        section_content_type = 'rows' if not isinstance(output[section], str) else 'characters'
-        logger.info(f'section {section} has {len(output[section])} {section_content_type}')
 
-    ipr_result = None
-    comments = {
-        'comments': summarize(
-            graphkb_conn,
-            alterations,
-            disease_name=kb_disease_match,
-            variants=expression_variants + copy_variants + structural_variants + small_mutations,
-        )
-    }
     output = clean_unsupported_content(output)
-    report_id = None
+    ipr_result = None
+
     if ipr_upload:
         try:
             logger.info(f'Uploading to IPR {ipr_conn.url}')
             ipr_result = ipr_conn.upload_report(output)
-            report_id = ipr_result['ident']
             logger.info(ipr_result)
-            logger.info('adding analyst comments')
             output.update(ipr_result)
         except Exception as err:
             logger.error(f"ipr_conn.upload_report failed: {err}", exc_info=True)
@@ -324,13 +328,6 @@ def create_report(
             logger.info(f'Writing IPR upload json to: {output_json_path}')
             with open(output_json_path, 'w') as fh:
                 fh.write(json.dumps(output))
-
-    if report_id:
-        try:
-            ipr_conn.set_analyst_comments(report_id, comments)
-            logger.info(f'report {report_id} was annotated with generated comments')
-        except Exception as err:
-            logger.error(f"ipr_conn.set_analyst_comments failed: {err}", exc_info=True)
     logger.info(f'made {graphkb_conn.request_count} requests to graphkb')
     logger.info(f'average load {int(graphkb_conn.load or 0)} req/s')
     return output
